__all__ = ["agriquest_service", "analytics_processor_service", "analytics_fabric_service",
           "master_data_management_service", "weather_service", "vegetation_time_series_service",
           "map_product_service"]
